<?php
/**
 * Created by PhpStorm.
 * User: ep
 * Date: 12/23/15
 * Time: 6:17 PM
 */


namespace qfq;

use qfq;

require_once(__DIR__ . '/../qfq/store/Config.php');
require_once(__DIR__ . '/../qfq/Constants.php');
require_once(__DIR__ . '/../qfq/helper/KeyValueStringParser.php');

const PAGEID = 'id';
const PARAM_GET = 'paramGet';
const URL_PRINT = 'urlPrint';

/**
 * Create a temporary file.
 *
 * @return string
 */
function createEmptyFile() {
    return tempnam(sys_get_temp_dir(), "webkitPrintPdf");
}

/**
 * Set HTML Header to initiate PDF download.
 *
 * @param $filename
 */
function setHeader($filename) {

    header("Content-Disposition: inline; filename=\"$filename\"");
    header("Content-Type: application/pdf");
    header("Content-Transfer-Encoding: binary");
}

/**
 * Read QFQ config. Only SYSTEM_BASE_URL_PRINT and SYSTEM_WKHTMLTOPDF will be used.
 * Check and get all clean _GET Parameter. Build a URL based on SYSTEM_BASE_URL_PRINT and the delivered URL params.
 *
 * @return array
 * @throws UserFormException
 * @throws \exception
 */
function init() {
    $cfg = new Config();

    $config = $cfg->readConfig('');
    $param = readCleanGetParam();

    if (!isset($config[SYSTEM_BASE_URL_PRINT]) || $config[SYSTEM_BASE_URL_PRINT] == '') {
        throw new \exception(CONFIG_INI . ' - Missing ' . SYSTEM_BASE_URL_PRINT);
    }

    if (!isset($config[SYSTEM_WKHTMLTOPDF]) || $config[SYSTEM_WKHTMLTOPDF] == '') {
        throw new \exception(CONFIG_INI . ' - Missing ' . SYSTEM_WKHTMLTOPDF);
    }

    if (!is_executable($config[SYSTEM_WKHTMLTOPDF])) {
        throw new \exception(CONFIG_INI . ' - ' . SYSTEM_WKHTMLTOPDF . '=' . $config[SYSTEM_WKHTMLTOPDF] . ' - not found or not executable.');
    }

    if (!isset($param[PAGEID]) || $param[PAGEID] == '') {
        throw new \exception("Missing GET Parameter '" . PAGEID . "'.");
    }

    $setup = array();
    $setup[URL_PRINT] = $config[SYSTEM_BASE_URL_PRINT] . '/?' . KeyValueStringParser::unparse($param, '=', '&');
    $setup[PAGEID] = $param[PAGEID];
    $setup[SYSTEM_WKHTMLTOPDF] = $config[SYSTEM_WKHTMLTOPDF];

    return $setup;
}

;

/**
 * Return an array with GET params who are clean - they do not violate $pattern.
 *
 * @return array
 */
function readCleanGetParam() {

    $param = array();
    $pattern = '^[\-_\.,;:\/a-zA-Z0-9]*$'; // ':alnum:' does not work here in FF

    foreach ($_GET as $key => $value) {
        if (preg_match("/$pattern/", $value) === 1) {
            $param[$key] = $value;
        }
    }

    return $param;
}

/**
 * @param array $setup
 * @throws \exception
 */
function page2pdf($setup) {

    $rc = 0;
    $file = createEmptyFile();

    $cmd = $setup[SYSTEM_WKHTMLTOPDF] . " '" . $setup[URL_PRINT] . "' " . $file . " > $file.log 2>&1";

    $line = system($cmd, $rc);

    if ($rc == 0) {
        setHeader('print.' . $setup[PAGEID] . '.pdf');
        @readfile($file);
        @unlink($file);
        @unlink($file . '.log');
        exit; // Do an extremely hard exit here to make sure there are no more additional bytes sent (makes the delivered PDF unusable).
    } else {
        throw new \exception("Error [RC=$rc] $line: $cmd");
    }
}


/**
 * Main
 */
try {
    $setup = init();

    page2pdf($setup);

} catch (\Exception $e) {
    echo "Exception: " . $e->getMessage();
}
